import { normalizePath } from '@ngtools/webpack/src/ivy/paths.js';
import { readFileSync } from 'node:fs';
import { compileAnalogFile } from './authoring/analog.js';
import { TEMPLATE_TAG_REGEX } from './authoring/constants.js';
import { createRequire } from 'node:module';
const require = createRequire(import.meta.url);
export function augmentHostWithResources(host, transform, options = {}) {
    const ts = require('typescript');
    const resourceHost = host;
    const baseGetSourceFile = resourceHost.getSourceFile.bind(resourceHost);
    if (options.supportAnalogFormat) {
        resourceHost.getSourceFile = (fileName, languageVersionOrOptions, onError, ...parameters) => {
            if (fileName.includes('.analog') || fileName.includes('.agx.ts')) {
                const contents = readFileSync(fileName.replace('.analog.ts', '.analog').replace('.agx.ts', '.agx'), 'utf-8');
                const source = compileAnalogFile(fileName, contents, options.isProd);
                return ts.createSourceFile(fileName, source, languageVersionOrOptions, onError, ...parameters);
            }
            return baseGetSourceFile.call(resourceHost, fileName, languageVersionOrOptions, onError, ...parameters);
        };
        const baseReadFile = resourceHost.readFile;
        resourceHost.readFile = function (fileName) {
            if (fileName.includes('virtual-analog:')) {
                const filePath = fileName.split('virtual-analog:')[1];
                const fileContent = baseReadFile.call(resourceHost, filePath) ||
                    'No Analog Markdown Content Found';
                // eslint-disable-next-line prefer-const
                const templateContent = TEMPLATE_TAG_REGEX.exec(fileContent)?.pop()?.trim() || '';
                return templateContent;
            }
            return baseReadFile.call(resourceHost, fileName);
        };
        const fileExists = resourceHost.fileExists;
        resourceHost.fileExists = function (fileName) {
            if (fileName.includes('virtual-analog:') &&
                !fileName.endsWith('analog.d') &&
                !fileName.endsWith('agx.d')) {
                return true;
            }
            return fileExists.call(resourceHost, fileName);
        };
    }
    resourceHost.readResource = async function (fileName) {
        const filePath = normalizePath(fileName);
        let content = this.readFile(filePath);
        if (content === undefined) {
            throw new Error('Unable to locate component resource: ' + fileName);
        }
        if (fileName.includes('virtual-analog:')) {
            for (const transform of options.markdownTemplateTransforms || []) {
                content = await transform(content, fileName);
            }
        }
        return content;
    };
    resourceHost.transformResource = async function (data, context) {
        // Only style resources are supported currently
        if (context.type !== 'style') {
            return null;
        }
        if (options.inlineStylesExtension) {
            // Resource file only exists for external stylesheets
            const filename = context.resourceFile ??
                `${context.containingFile.replace(/(\.analog)?\.ts$/, (...args) => {
                    // NOTE: if the original file name contains `.analog`, we turn that into `-analog.css`
                    if (args.includes('.analog') || args.includes('.agx')) {
                        return `-analog.${options?.inlineStylesExtension}`;
                    }
                    return `.${options?.inlineStylesExtension}`;
                })}`;
            let stylesheetResult;
            try {
                stylesheetResult = await transform(data, `${filename}?direct`);
            }
            catch (e) {
                console.error(`${e}`);
            }
            return { content: stylesheetResult?.code || '' };
        }
        return null;
    };
}
//# sourceMappingURL=host.js.map